package class9_1_MultithreadPrimeApp_start;

import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;
import java.util.List;

/**
 * This class is NOT yet multithreaded
 */
public class PrimeApp extends JFrame {
    private Invoker invoker = new Invoker();

    private List<Long> primes = new ArrayList<>();
    private final JLabel statusLabel;

    public static void main(String[] ignored) {
        new PrimeApp();
    }

    public PrimeApp() {
        setSize(800,600);
        setLayout(new FlowLayout());
        setDefaultCloseOperation(EXIT_ON_CLOSE);

        statusLabel = new JLabel();
        statusLabel.setText("Latest primes:");
        add(statusLabel);

        JMenuBar bar = new JMenuBar();
        JMenu menu = new JMenu("Actions");
        JMenuItem item = new JMenuItem("Start finding primes");
        item.addActionListener(e -> startPrimes());
        menu.add(item);
        item = new JMenuItem("Stop finding primes");
        item.addActionListener(e -> stopPrimes());
        menu.add(item);
        bar.add(menu);
        setJMenuBar(bar);

        setVisible(true);
    }

    public void startPrimes() {
        findPrimes();
    }

    public void stopPrimes() {
        System.out.println("Does nothing..");
    }

    /**
     * This class already includes an invoker with an invoke
     * method that automatically runs tasks on a separate thread.
     * Invoker uses Runnable for its Command interface.
     *
     * TODO: When findPrimes is called, it should find all the primes
     * from 100000000 to 100000399 and then stop.
     *
     * a. Create commands to find all primes in ranges
     * of 100 numbers. For example,
     * one command might find the primes
     * from 100000000 through 100000099, the next one
     * from 100000100 through 100000199, etc.
     *
     * b. findPrimes should create four such commands to perform this work.
     *
     * c. The GUI should be responsive throughout this process, and
     * interaction with the GUI should be thread-safe.
     */
    public void findPrimes() {
        for(long candidate = 100000000; candidate<Long.MAX_VALUE ; candidate++) {
            if(isPrime(candidate)) {
                primes.add(candidate);
                String text = "Latest prime: " + candidate;
                statusLabel.setText(text);
                System.out.println(text);
            }
        }
    }

    /**
     * Slow O((2^b)) or worse algorithm (where b is the number of bits in candidate)
     *
     * TODO: Implement Miller-Rabin Primality test to get O(n^12) or better performance.
     * http://en.wikipedia.org/wiki/Miller%E2%80%93Rabin_primality_test
     * (Why nine characters for one dash?
     *   Unicode character  Oct     Dec	    Hex	    HTML
     *            en dash   020023	8211	0x2013	&ndash;   )
     *
     * @param candidate candidate to test
     * @return true if candidate is prime.
     */
    private boolean isPrime(long candidate) {
        boolean isPrime = true;
        for(int divisor = 2; divisor < candidate && isPrime; divisor++) {
            if(candidate % divisor == 0) {
                isPrime = false;
            }
        }
        return isPrime;
    }
}
